import 'package:flutter/material.dart';
import 'dart:convert';

String gradientToString(Gradient currentGradient) {
  Alignment alignment1;
  Alignment alignment2;

  if (currentGradient is LinearGradient) {
    alignment1 = currentGradient.begin;
    alignment2 = currentGradient.end;
    return "{"+
        '''"stops": ''' +
        listToString(currentGradient.stops)+
        ''', "colors": ''' +
        listToString(currentGradient.colors, func: colorToString)+
        ''', "alignment1": ''' +
        alignmentToString(alignment1) +
        ''', "alignment2": ''' +
        alignmentToString(alignment2) +
        ''', "tileMode": ''' +
        stringToString(currentGradient.tileMode.toString()) +
        "}";
  } else if (currentGradient is RadialGradient){
    alignment1 = currentGradient.center;
    double radius = currentGradient.radius;
    return  "{"+
        '''"stops": ''' +
        listToString(currentGradient.stops)+
        ''', "colors": ''' +
        listToString(currentGradient.colors, func: colorToString)+
        ''', "alignment1": ''' +
        alignmentToString(alignment1) +
        ''', "radius": ''' +
        radius.toString() +
        ''', "tileMode": ''' +
        stringToString(currentGradient.tileMode.toString()) +
        "}";
  }else if (currentGradient is SweepGradient){
    alignment1 = currentGradient.center;
    double startAngle =currentGradient.startAngle;
    double endAngle = currentGradient.endAngle;
    return  "{"+
        '''"stops": ''' +
        listToString(currentGradient.stops)+
        ''', "colors": ''' +
        listToString(currentGradient.colors, func: colorToString)+
        ''', "alignment1": ''' +
        alignmentToString(alignment1) +
        ''', "startAngle": ''' +
        startAngle.toString() +
        ''', "endAngle": ''' +
        endAngle.toString() +
        ''', "tileMode": ''' +
        stringToString(currentGradient.tileMode.toString()) +
        "}";
  }
}

String imageToString(DecorationImage decorationImage) {
  String rst = "{"+
      '''"fit": ''' +
      stringToString(decorationImage.fit.toString()) +
      ''', "alignment": ''' +
      alignmentToString(decorationImage.alignment);
  String temp = decorationImage.colorFilter.toString();
  List<String> filtermode =
  temp.substring(temp.indexOf("("), temp.lastIndexOf((")"))).split(",");
  String colorString = filtermode[0];
  colorString = colorString.substring(
      colorString.lastIndexOf("(") + 3, colorString.indexOf(")"));
  rst += ''', "color": ''' +
      stringToString(colorString) +
      ''', "mode": ''' +
      stringToString(filtermode[1]) +
      ''', "url": ''' +
      stringToString((decorationImage.image as NetworkImage).url.toString()) +
      "}";
  return rst;
}

String boxDecorationToString(BoxDecoration boxDecoration) {
  String rst = '''{
    "color": ''' +
      colorToString(boxDecoration.color ?? Colors.white);
  if (boxDecoration.gradient != null) {
    rst += ''', "gradient": ''' + gradientToString(boxDecoration.gradient);
  }
  if (boxDecoration.image != null) {
    rst += ''', "image": ''' + imageToString(boxDecoration.image);
  }
  return rst +
      '''
    }''';
}

String textStyleToString(TextStyle textStyle) {
  return "{"+
      '''"fontFamily": ''' +
      stringToString(textStyle.fontFamily) +
      ''', "color": ''' +
      colorToString(textStyle.color) +
      ''', "backgroundColor": ''' +
      colorToString(textStyle.backgroundColor) +
      ''', "fontWeight": ''' +
      stringToString(textStyle.fontWeight.toString()) +
      ''', "fontStyle": ''' +
      stringToString(textStyle.fontStyle.toString()) +
      ''', "decoration": ''' +
      stringToString(textStyle.decoration.toString()) +
      ''', "decorationColor": ''' +
      colorToString(textStyle.decorationColor) +
      ''', "decorationStyle": ''' +
      stringToString(textStyle.decorationStyle.toString()) +
      ''', "decorationThickness": ''' +
      textStyle.decorationThickness.toString() +
      ''', "height": ''' +
      textStyle.height.toString() +
      ''', "letterSpacing": ''' +
      textStyle.letterSpacing.toString() +
      "}";
}

String stringToString(String str) {
  return "\"" + str + "\"";
}

String colorToString(Color color) {
  return stringToString(color.value.toRadixString(16));
}

String axisToString(Axis axis) {
  return stringToString(axis.toString());
}

String alignmentToString(Alignment alignment) {
  return stringToString(alignment.toString());
}

String textAlignToString(TextAlign alignment) {
  return stringToString(alignment.toString());
}

String mainAxisAlignmentToString(MainAxisAlignment alignment) {
  return stringToString(alignment.toString());
}

String crossAxisAlignmentToString(CrossAxisAlignment alignment) {
  return stringToString(alignment.toString());
}

String wrapAlignmentToString(WrapAlignment alignment) {
  return stringToString(alignment.toString());
}

String wrapCrossAlignmentToString(WrapCrossAlignment alignment) {
  return stringToString(alignment.toString());
}

String listToString(List<dynamic> list, {Function func}) {
  String rst= ''' [''';
  if(list.length>0) {
    if(func == null) {
      for (int i = 0; i < list.length; i++) {
        rst += list[i].toString() + ", ";
      }
    }else{
      for (int i = 0; i < list.length; i++) {
        rst += func(list[i]) + ", ";
      }
    }
    rst = rst.substring(0, rst.length - 2);
  }
  rst+= ''']''';
  return rst;
}

String variableToString(String name, dynamic variable, {Function func}) {
  if(variable==null) {
    return "";
  }else if (func==null){
    return ", \""+ name+"\":" +
        variable.toString();
  }
  return ", \""+ name+"\":" +
      func(variable);
}

String styleToString(dynamic style) {
  return json.encode(style);
}