import 'ParsingUtils.dart';
import 'ConvertUtils.dart';
import 'package:flutter/material.dart';
import 'CssUnit.dart';
import 'dart:convert';

class Style {
  bool visible;

  Alignment alignment;
  Length width;
  Length height;

  Length marginLeft;
  Length marginRight;
  Length marginTop;
  Length marginBottom;
  Length paddingLeft;
  Length paddingRight;
  Length paddingTop;
  Length paddingBottom;

  BoxDecoration backgroundDecoration;
  Color borderColor;
  double borderThickness;
  double elevation;
  DynamicShadow shadow;
  String shape;

  double rotationX;
  double rotationY;
  double rotationZ;
  double skewX;
  double skewY;
  double translationX;
  double translationY;

  //text related (works only for text and icon widgets)
  Alignment childAlignment;
  Length fontSize;
  DynamicShadow textShadow;
  TextStyle textStyle;
  TextAlign textAlign;
  Gradient textGradient;

  Style({
    this.visible,
    this.alignment,
    this.width = const Length(0, unit: LengthUnit.auto),
    this.height = const Length(0, unit: LengthUnit.auto),
    this.marginLeft = const Length(0),
    this.marginRight = const Length(0),
    this.marginTop = const Length(0),
    this.marginBottom = const Length(0),
    this.paddingLeft = const Length(0),
    this.paddingRight = const Length(0),
    this.paddingTop = const Length(0),
    this.paddingBottom = const Length(0),
    this.backgroundDecoration,
    this.borderColor,
    this.borderThickness,
    this.elevation,
    this.shadow,
    this.shape,

    this.rotationX,
    this.rotationY,
    this.rotationZ,
    this.skewX,
    this.skewY,
    this.translationX,
    this.translationY,
    this.childAlignment,
    this.fontSize,
    this.textShadow,
    this.textStyle,
    this.textAlign,
    this.textGradient,
  });

  Style.fromJson(String css) {
    var map = jsonDecode(css);

    visible = parseBoolInMap(map, "visible", defaultValue: true);

    width = parseLengthInMap(map, "width",
        defaultValue: Length(0, unit: LengthUnit.auto));
    height = parseLengthInMap(map, "height",
        defaultValue: Length(0, unit: LengthUnit.auto));
    alignment = parseAlignmentInMap(map, "alignment");

    marginLeft = parseLengthInMap(map, "marginLeft", defaultValue: Length(0));
    marginRight = parseLengthInMap(map, "marginRight", defaultValue: Length(0));
    marginTop = parseLengthInMap(map, "marginTop", defaultValue: Length(0));
    marginBottom =
        parseLengthInMap(map, "marginBottom", defaultValue: Length(0));

    paddingLeft = parseLengthInMap(map, "paddingLeft", defaultValue: Length(0));
    paddingRight =
        parseLengthInMap(map, "paddingRight", defaultValue: Length(0));
    paddingTop = parseLengthInMap(map, "paddingTop", defaultValue: Length(0));
    paddingBottom =
        parseLengthInMap(map, "paddingBottom", defaultValue: Length(0));

    backgroundDecoration = parseBoxDecorationInMap(map, "backgroundDecoration");
    borderColor = parseColorInMap(map, "borderColor");
    borderThickness = parseDoubleInMap(map, "borderThickness");
    elevation = parseDoubleInMap(map, "elevation");
    shadow = parseDynamicShadowInMap(map, "shadow");
    shape = parseStringInMap(map, "shape");

    rotationX = parseDoubleInMap(map, "rotationX");
    rotationY = parseDoubleInMap(map, "rotationY");
    rotationZ = parseDoubleInMap(map, "rotationZ");
    skewX = parseDoubleInMap(map, "skewX");
    skewY = parseDoubleInMap(map, "skewY");
    translationX = parseDoubleInMap(map, "translationX");
    translationY = parseDoubleInMap(map, "translationY");

    textAlign = parseTextAlignInMap(map, "textAlign");
    fontSize = parseLengthInMap(map, "fontSize");
    textShadow = parseDynamicShadowInMap(map, "textShadow");
    textStyle = parsetextStyleInMap(map, "textStyle");
    textGradient = parseGradientInMap(map, "textGradient");

  }

  String toJson() {
    String rst = variableToString("visible", visible) +
        variableToString("alignment", alignment, func: alignmentToString) +
        variableToString("width", width) +
        variableToString("height", height) +
        variableToString("marginLeft", marginLeft) +
        variableToString("marginRight", marginRight) +
        variableToString("marginTop", marginTop) +
        variableToString("marginBottom", marginBottom) +
        variableToString("paddingLeft", paddingLeft) +
        variableToString("paddingRight", paddingRight) +
        variableToString("paddingTop", paddingTop) +
        variableToString("padding Bottom", paddingBottom) +
        variableToString("backgroundDecoration", backgroundDecoration,
            func: boxDecorationToString) +
        variableToString("borderColor", borderColor, func: colorToString) +
        variableToString("borderThickness", borderThickness) +
        variableToString("shape", shape, func: stringToString) +
        variableToString("elevation", elevation) +
        variableToString("shadow", shadow)+
        variableToString("rotationX", rotationX)+
        variableToString("rotationY", rotationY)+
        variableToString("rotationZ", rotationZ)+
        variableToString("skewX", skewX)+
        variableToString("skewY", skewY)+
        variableToString("translationX", translationX)+
        variableToString("translationY", translationY)+
        variableToString("childAlignment", childAlignment, func: alignmentToString)+
        variableToString("fontSzie", fontSize)+
        variableToString("textShadow", textShadow)+
        variableToString("textAlign", textAlign, func: textAlignToString)+
        variableToString("textStyle", textStyle, func: textStyleToString)+
        variableToString("textGradient", textGradient, func: gradientToString);
    rst = "{" + rst.replaceFirst(",", "") + "}";
    return rst;
  }
}

dynamic chooseFrom(dynamic input, double screenWidth) {
  dynamic rst;
  if(input is Map) {
    double tempMaxWidth=double.infinity;
    input.forEach((key, value) {
      double allowedMaxWidth=key is double ? key : double.tryParse(key)??double.infinity;
      if (screenWidth<allowedMaxWidth && tempMaxWidth>=allowedMaxWidth) {
        rst=value;
        tempMaxWidth=allowedMaxWidth;
      }
    });
  }else{
    rst=input;
  }

  return rst;
}